package com.tzld.piaoquan.content.understanding.util;

import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.TimeUnit;
import java.util.function.LongSupplier;
import java.util.function.Predicate;
import java.util.function.Supplier;

@Slf4j
public class RetryUtil {

    private static final Integer MAX_RETRIES = 3;

    /**
     * 重试
     *
     * @param action
     * @param actionName
     * @param <T>
     * @return
     */
    public static <T> T executeWithRetry(Supplier<T> action, String actionName) {
        return executeWithRetry(action, MAX_RETRIES, actionName);
    }

    /**
     * 重试
     *
     * @param action
     * @param maxRetries
     * @param actionName
     * @param <T>
     * @return
     */
    public static <T> T executeWithRetry(Supplier<T> action, Integer maxRetries, String actionName) {
        for (int i = 1; i <= maxRetries; i++) {
            try {
                return action.get();
            } catch (Exception e) {
                log.error("{} failed on attempt {}: {}", actionName, i, e.getMessage(), e);
                if (i < maxRetries) {
                    log.info("{} will be retried (attempt {}/{})", actionName, i + 1, maxRetries);
                } else {
                    log.error("{} failed after {} retries due to non-IO error.", actionName, maxRetries);
                    throw new RuntimeException(actionName + " failed after multiple retries due to error.", e);
                }
            }
        }
        return null;
    }
}
